<?php

declare(strict_types=1);

namespace Drupal\vipps_recurring_payments_webform\Repository;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\webform\WebformSubmissionInterface;
use League\Container\Exception\NotFoundException;

/**
 * Class Webform Submission Repository.
 *
 * @package Drupal\vipps_recurring_payments_webform\Repository
 */
class WebformSubmissionRepository {

  /**
   * Storage interface.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $storage;

  /**
   * WebformSubmissionRepository constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->storage = $entityTypeManager->getStorage('webform_submission');
  }

  /**
   * Get submission by id.
   *
   * @param int $id
   *   Submission id.
   *
   * @return \Drupal\webform\WebformSubmissionInterface
   *   Webform submission.
   */
  public function getById(int $id): WebformSubmissionInterface {
    /** @var \Drupal\webform\WebformSubmissionInterface $submission */
    $submission = $this->storage->load($id);

    if (is_null($submission)) {
      throw new NotFoundException();
    }

    return $submission;
  }

  /**
   * Set status from agreement.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $entity
   *   Webform submission.
   * @param string $status
   *   Agreement status.
   */
  public function setStatus(WebformSubmissionInterface $entity, string $status): void {
    $entity->setElementData('agreement_status', $status);
    $entity->resave();
  }

  /**
   * Set agreement id.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $entity
   *   Webform submission.
   * @param string $agreementId
   *   Agreement id.
   */
  public function setAgreementId(WebformSubmissionInterface $entity, string $agreementId): void {
    $entity->setElementData('agreement_id', $agreementId);
    $entity->resave();
  }

}
